#!/bin/bash
set -euo pipefail
WD=$(mktemp -d)
cleanup (){
	rm -rf "${WD}"
}
trap cleanup EXIT

cd "${WD}"

CROPSWAPARCHIVE=cropswap-archive@shandan.one
ITER_SUBS=/opt/cropswap/gdbm-iter-subscriptions.py
MIME_INJECT=/opt/cropswap/mime-inject.py
DELIVERED=false
SENDMAIL="/usr/sbin/sendmail"
if ((DEBUG != 0)); then
	SENDMAIL="$DEBUGMAIL"
fi

mk_subject(){
	subject="$(reformail -c -x "Subject:")"
	if [ "${subject/\[${SUBJECT}\]/}" == "${subject}" ]; then
		subject="[${SUBJECT}] ${subject## }"
	fi
	echo "${subject}"
}
mk_headers(){
	subject="$(reformail \
		-X "Date:" \
		-X "MIME-Version:" \
		-X "Subject:" \
		-X "Content-Type:" \
	| tee >(
		reformail \
			-U "MIME-Version:" \
			-U "Content-Type:" \
		| mk_list_headers > headers
	) | mk_subject)"
	reformail -I "Subject: $subject" < headers
}
mk_list_headers(){
	if [ -n "$REPLYTO" ]; then 
		reformail -I "Reply-To: $REPLYTO"
		return
	fi
	reformail \
		-I "List-Id: $MAILGROUP" \
		-I "List-Unsubscribe: <mailto:$MAILGROUP?subject=[UNSUBSCRIBE]>" \
		-I "List-Subscribe: <mailto:$MAILGROUP?subject=[SUBSCRIBE]>" \
		-I "List-Post: <mailto:$MAILGROUP>" \
		-I "List-Unsubscribe-Post: List-Unsubscribe=One-Click"
}
tee >(mk_headers > mail) | "${MIME_INJECT}" "${REPLYTO:-$MAILGROUP}" | reformime -s1 -e > body
cat body >> mail

declare -a SUBSCRIBED
mapfile -t SUBSCRIBED < <("${ITER_SUBS}" "${MLIST}" "${SLIST}" "${NOFORWARD}" "${TO}")

for addr in "${SUBSCRIBED[@]}"; do
	echo "Forwarding mail: "
	echo "From: ${FROM} <${MAILGROUP}>"
	echo "To: $addr"
	"$SENDMAIL" -F "${FROM}" -f "${MAILGROUP}" "$addr" < mail
	DELIVERED=true
done
if [ "${MAILGROUP}" == "${MEMBERGROUP:-x}" ]
then
	reformail \
		-I "List-Id:" \
		-I "List-Unsubscribe:" \
		-I "List-Subscribe:" \
		-I "List-Post:" \
		-I "List-Unsubscribe-Post:" \
		-I "To: ${TOALIAS:-} <${MAILGROUP}>" \
	< mail | "$SENDMAIL" -F "${FROM}" -f "${MAILGROUP}" "${CROPSWAPARCHIVE}"
fi

"$DEBUGMAIL" "delivered:$DELIVERED" < mail || true
if ! $DELIVERED; then
	exit "$UNHANDLED"
else
	exit 0
fi
